/*
  $Author: kinch $
  $Id: suadi.h,v 1.22 1995/11/02 20:18:52 kinch Exp $
  $Source: /usr/src/usr.local/security/suadi/RCS/suadi.h,v $
  $Locker:  $

  Original work August 1994 by Dave Kinchlea <kinch@julian.uwo.ca>
  with Kudos to Reg Quinton <reggers@julian.uwo.ca> for the original
  idea and much help along the way.

  Copyright (c) 1994 The University of Western Ontario.  All rights
  reserved.  Redistribution is permitted provided that this notice is
  preserved and that due credit is given to the authors as well as The
  University of Western Ontario.

  This is the suadi header file. Any and all architectural modifications 
  should be made in this file (if at all possible) or better yet
  with 'make CFLAGS=-DVAR=value'.

*/

#ifndef SUADI_H
#define SUADI_H

#define USAGE fprintf(stderr, "usage: %s [user [-c command]]\nor\n%s -p\n\
\t %s: alone provides a standard root shell\n\
\t %s user: provides a shell under user\'s uid\n\
\t %s user -c command: will pass the \"-c command\" to the user\'s shell\n\
\t %s -p: allows an existing %s user to change their password\n",\
progname,progname,progname,progname,progname,progname,progname)


/* All too often, there is a necessity to differentiate between system v and 
   bsd environments. There is no standard way to determine this, we EXPECT
   one of -DSYSV or -DBSD to be included, we will barf otherwise. 

NOTE: Not all compilers can deal with the #error preprocessor symbol */

#ifndef   SYSV 
#ifndef   BSD
#error "One of SYSV or BSD must be defined"
#endif /* SYSV */
#endif /* BSD */

/* GCC defines __STRICT_ANSI__, __STDC__ is the proper way to do this */

#ifdef __STRICT_ANSI__
#ifndef __STDC__
#define __STDC__
#endif
#endif

/* Necessary Include files */

#include <stdio.h>
/* The following useful extension is used. */
extern FILE     *fdopen(int, const char *);
#ifndef __mips__
#include <unistd.h>
#endif
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifdef __mips__
#include <posix/sys/termios.h>
#else
#include <termios.h>
#endif
#include <pwd.h>
#include <syslog.h>
#include <sys/file.h>
#include <fcntl.h>
#include <time.h>
#include <sys/wait.h>
#include <errno.h>

#ifdef SYSV
#include <sys/stropts.h>     /* This is the streams stuff for SYSV pty */
#endif

#ifdef __aix__
#include <sys/select.h>
#include <sys/ioctl.h>
#define SLEEPTIME 3    /* time in seconds to sleep: This is necessary to subvert
			  a bad timing bug shown (so far) only in AIX 3.2.5 systems
			  The suspicion is that the code in dooutput() which is
			  simply a while read >= 0 returns -1 because the other
			  end of the pipe is not setup yet. You may have to adjust
			  SLEEPTIME depending on how busy your system is (how many
			  failed pseudo tty lookups). */
			  
#endif

#ifdef HAS_CRYPT_H
#include <crypt.h>
#else
#define NO_CRYPT_PROTO
#endif

/* Helpful macros for output */

#ifndef	debug
#define debug 0 /* (non-zero is on) */
#endif

/* Useful CONSTANTS */

/* UIDs below MAGIUID will, when enoking suadi with a -c command option,
   not use logging UNLESS the -l flag is given */
#define MAGICUID 10

#define MAX_SYSLOG_LEN 256 /* Allow room for syslog formating */

/* Allow for a different AUTHFILE to be given at compile time, else
use one in current directory if being compiled with debug (its easier
that way) */

#ifndef AUTHFILE
#if debug == 0
#define AUTHFILE "/etc/suadi.passwd"
#else
#define AUTHFILE "suadi.passwd"
#endif  /* ifdef debug */
#endif  /* ifndef AUTHFILE */

/* BUFSIZ is a system constant, we use BUFSIZE so you can freely change it */

#define BUFSIZE BUFSIZ

/* TTY modes */

#ifndef LFLAG
#define LFLAG 0     /* We want RAW mode */
#endif 
#ifndef MIN_CHAR_NUM
#define MIN_CHAR_NUM 1 /* minimum number of characters to read (stdin) */
#endif 
#ifndef MIN_CHAR_TIME
#define MIN_CHAR_TIME 10 /* Intercharacter timeout value */
#endif

/* The following are for syslog calls. If debug is defined, the
   default syslog values are LOG_LOCAL3:LOG_DEBUG:(LOG_PID|LOG_NOWAIT) if
   debug is not defined, the default values are:
   LOG_AUTH:LOG_INFO:(LOG_PID|LOG_NOWAIT). These can, of course, be
   changed at compile time via appropriate -D statements. */

#ifndef SYSLOG_FACILITY
#if debug == 0
#define SYSLOG_FACILITY LOG_AUTH
#else
#define SYSLOG_FACILITY LOG_LOCAL3
#endif /* ifdef debug */
#endif /* ifndef SYSLOG_FACILITY */

#ifndef SYSLOG_OPTIONS
#define SYSLOG_OPTIONS LOG_PID|LOG_NOWAIT
#endif

#ifndef SYSLOG_PRIORITY
#define SYSLOG_PRIORITY LOG_DEBUG
#endif   /* ifndef SYSLOG_PRIORITY */


/* Some older systems don't have strerror but usually provide sys_errlist */

#ifdef NO_STRERROR
extern char *sys_errlist[];
extern int sys_nerr;
#define strerror(x) (sys_nerr >= errno? sys_errlist[x] : "Unknown error number")
#endif

/* Return Values */

#define TRUE 1
#define FALSE 0
#define SUCCESS  0
#define BAD_PASS 1
#define BAD_USER 2
#define SYSFAIL  3
#define LOCKED   4
#ifndef NAME_MAX
#define NAME_MAX 256   /* Max number of characters in a program name */
#endif

/* default root uid, gid, shell and directory, 
   overrriden by entry in AUTHFILE */

#define ROOTUID   0
#define ROOTGID   0
#define ROOTNAME  "root"
#define ROOTSHELL "/bin/sh"
#define ROOTDIR   "/"

/* Function Prototype declaration section */


#if !defined __STDC__ && !defined __mips__
#define void
#else
void doinput(void);
void finish(int);
void dooutput(void);
void doshell(int, char **);
void fixtty(void);
void fail(void);
void done(void);
void getmaster(void);
void getslave(void);
int authenticate_user(const char *);
void logit(char *,int);
int change_passwd(const char *);

/* libpass does not yet have a header file, so: */
int configure(char *);
int crack(char *, char *, char *, int);

#ifdef NO_PWD_PROTO
struct passwd *fgetpwent(FILE *);
#endif

#ifdef NO_GETPASS_PROTO
char *getpass(const char *);
#endif

#ifdef NO_CRYPT_PROTO
char *crypt(const char *, const char *);
#endif

#endif /*__STDC__ && __mips__*/

/* Dont get too confused here, getpwuid returns a struct passwd *, 
   I am simply using that pointer to get at the login name. */

#define Getlogin() (getpwuid(getuid()))->pw_name  

#ifdef __mips__
struct passwd *getpwuid(struct passwd *);
struct passwd *getuid(void);
#endif

#endif /* SUADI_H */
